@@ -25,8 +25,8 @@ public:
     OverlapAnalysis();
     ~OverlapAnalysis();
 
-    static OverlapResult analyze(Sequence&  r1, Sequence&  r2, int overlapDiffLimit = 5, int overlapRequire=30);
-    static OverlapResult analyze(Read* r1, Read* r2, int overlapDiffLimit = 5, int overlapRequire=30);
+    static OverlapResult analyze(Sequence&  r1, Sequence&  r2, int diffLimit, int overlapRequire, double diffPercentLimit);
+    static OverlapResult analyze(Read* r1, Read* r2, int diffLimit, int overlapRequire, double diffPercentLimit);
     static Read* merge(Read* r1, Read* r2, OverlapResult ov);
 
 public:
