@@ -28,6 +28,10 @@ PairEndProcessor::PairEndProcessor(Options* opt){
     memset(mInsertSizeHist, 0, sizeof(long)*isizeBufLen);
     mLeftWriter =  NULL;
     mRightWriter = NULL;
+    mUnpairedLeftWriter =  NULL;
+    mUnpairedRightWriter = NULL;
+    mMergedWriter = NULL;
+    mFailedWriter = NULL;
 
     mDuplicate = NULL;
     if(mOptions->duplicate.enabled) {
@@ -44,6 +48,20 @@ PairEndProcessor::~PairEndProcessor() {
 }
 
 void PairEndProcessor::initOutput() {
+    if(!mOptions->unpaired1.empty())
+        mUnpairedLeftWriter = new WriterThread(mOptions, mOptions->unpaired1);
+
+    if(!mOptions->unpaired2.empty() && mOptions->unpaired2 != mOptions->unpaired1)
+        mUnpairedRightWriter = new WriterThread(mOptions, mOptions->unpaired2);
+
+    if(mOptions->merge.enabled) {
+        if(!mOptions->merge.out.empty())
+            mMergedWriter = new WriterThread(mOptions, mOptions->merge.out);
+    }
+
+    if(!mOptions->failedOut.empty())
+        mFailedWriter = new WriterThread(mOptions, mOptions->failedOut);
+
     if(mOptions->out1.empty())
         return;
     
@@ -61,6 +79,22 @@ void PairEndProcessor::closeOutput() {
         delete mRightWriter;
         mRightWriter = NULL;
     }
+    if(mMergedWriter) {
+        delete mMergedWriter;
+        mMergedWriter = NULL;
+    }
+    if(mFailedWriter) {
+        delete mFailedWriter;
+        mFailedWriter = NULL;
+    }
+    if(mUnpairedLeftWriter) {
+        delete mUnpairedLeftWriter;
+        mLeftWriter = NULL;
+    }
+    if(mUnpairedRightWriter) {
+        delete mUnpairedRightWriter;
+        mRightWriter = NULL;
+    }
 }
 
 void PairEndProcessor::initConfig(ThreadConfig* config) {
@@ -94,10 +128,22 @@ bool PairEndProcessor::process(){
 
     std::thread* leftWriterThread = NULL;
     std::thread* rightWriterThread = NULL;
+    std::thread* unpairedLeftWriterThread = NULL;
+    std::thread* unpairedRightWriterThread = NULL;
+    std::thread* mergedWriterThread = NULL;
+    std::thread* failedWriterThread = NULL;
     if(mLeftWriter)
         leftWriterThread = new std::thread(std::bind(&PairEndProcessor::writeTask, this, mLeftWriter));
     if(mRightWriter)
         rightWriterThread = new std::thread(std::bind(&PairEndProcessor::writeTask, this, mRightWriter));
+    if(mUnpairedLeftWriter)
+        unpairedLeftWriterThread = new std::thread(std::bind(&PairEndProcessor::writeTask, this, mUnpairedLeftWriter));
+    if(mUnpairedRightWriter)
+        unpairedRightWriterThread = new std::thread(std::bind(&PairEndProcessor::writeTask, this, mUnpairedRightWriter));
+    if(mMergedWriter)
+        mergedWriterThread = new std::thread(std::bind(&PairEndProcessor::writeTask, this, mMergedWriter));
+    if(mFailedWriter)
+        failedWriterThread = new std::thread(std::bind(&PairEndProcessor::writeTask, this, mFailedWriter));
 
     producer.join();
     for(int t=0; t<mOptions->thread; t++){
@@ -109,6 +155,14 @@ bool PairEndProcessor::process(){
             leftWriterThread->join();
         if(rightWriterThread)
             rightWriterThread->join();
+        if(unpairedLeftWriterThread)
+            unpairedLeftWriterThread->join();
+        if(unpairedRightWriterThread)
+            unpairedRightWriterThread->join();
+        if(mergedWriterThread)
+            mergedWriterThread->join();
+        if(failedWriterThread)
+            failedWriterThread->join();
     }
 
     if(mOptions->verbose)
@@ -136,15 +190,18 @@ bool PairEndProcessor::process(){
     cerr << "Read1 before filtering:"<<endl;
     finalPreStats1->print();
     cerr << endl;
-    cerr << "Read1 after filtering:"<<endl;
-    finalPostStats1->print();
-    cerr << endl;
     cerr << "Read2 before filtering:"<<endl;
     finalPreStats2->print();
+    cerr << endl;
     if(!mOptions->merge.enabled) {
+        cerr << "Read1 after filtering:"<<endl;
+        finalPostStats1->print();
         cerr << endl;
         cerr << "Read2 aftering filtering:"<<endl;
         finalPostStats2->print();
+    } else {
+        cerr << "Merged and filtered:"<<endl;
+        finalPostStats1->print();
     }
 
     cerr << endl;
@@ -220,6 +277,14 @@ bool PairEndProcessor::process(){
         delete leftWriterThread;
     if(rightWriterThread)
         delete rightWriterThread;
+    if(unpairedLeftWriterThread)
+        delete unpairedLeftWriterThread;
+    if(unpairedRightWriterThread)
+        delete unpairedRightWriterThread;
+    if(mergedWriterThread)
+        delete mergedWriterThread;
+    if(failedWriterThread)
+        delete failedWriterThread;
 
     if(!mOptions->split.enabled)
         closeOutput();
@@ -242,7 +307,11 @@ int PairEndProcessor::getPeakInsertSize() {
 bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
     string outstr1;
     string outstr2;
+    string unpairedOut1;
+    string unpairedOut2;
     string singleOutput;
+    string mergedOutput;
+    string failedOut;
     int readPassed = 0;
     int mergedCount = 0;
     for(int p=0;p<pack->count;p++){
@@ -285,7 +354,7 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
         }
         bool isizeEvaluated = false;
         if(r1 != NULL && r2!=NULL && (mOptions->adapter.enabled || mOptions->correction.enabled)){
-            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire);
+            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire, mOptions->overlapDiffPercentLimit/100.0);
             // we only use thread 0 to evaluae ISIZE
             if(config->getThreadId() == 0) {
                 statInsertSize(r1, r2, ov, frontTrimmed1, frontTrimmed2);
@@ -296,17 +365,23 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
             }
             if(mOptions->adapter.enabled) {
                 bool trimmed = AdapterTrimmer::trimByOverlapAnalysis(r1, r2, config->getFilterResult(), ov, frontTrimmed1, frontTrimmed2);
+                bool trimmed1 = trimmed;
+                bool trimmed2 = trimmed;
                 if(!trimmed){
                     if(mOptions->adapter.hasSeqR1)
-                        AdapterTrimmer::trimBySequence(r1, config->getFilterResult(), mOptions->adapter.sequence, false);
+                        trimmed1 = AdapterTrimmer::trimBySequence(r1, config->getFilterResult(), mOptions->adapter.sequence, false);
                     if(mOptions->adapter.hasSeqR2)
-                        AdapterTrimmer::trimBySequence(r2, config->getFilterResult(), mOptions->adapter.sequenceR2, true);
+                        trimmed2 = AdapterTrimmer::trimBySequence(r2, config->getFilterResult(), mOptions->adapter.sequenceR2, true);
+                }
+                if(mOptions->adapter.hasFasta) {
+                    AdapterTrimmer::trimByMultiSequences(r1, config->getFilterResult(), mOptions->adapter.seqsInFasta, false, !trimmed1);
+                    AdapterTrimmer::trimByMultiSequences(r2, config->getFilterResult(), mOptions->adapter.seqsInFasta, true, !trimmed2);
                 }
             }
         }
 
         if(config->getThreadId() == 0 && !isizeEvaluated && r1 != NULL && r2!=NULL) {
-            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire);
+            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire, mOptions->overlapDiffPercentLimit/100.0);
             statInsertSize(r1, r2, ov, frontTrimmed1, frontTrimmed2);
             isizeEvaluated = true;
         }
@@ -325,37 +400,42 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
 
         Read* merged = NULL;
         // merging mode
+        bool mergeProcessed = false;
         if(mOptions->merge.enabled && r1 && r2) {
-            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire);
+            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire, mOptions->overlapDiffPercentLimit/100.0);
             if(ov.overlapped) {
                 merged = OverlapAnalysis::merge(r1, r2, ov);
                 int result = mFilter->passFilter(merged);
                 config->addFilterResult(result, 2);
                 if(result == PASS_FILTER) {
-                    singleOutput += merged->toString();
+                    mergedOutput += merged->toString();
                     config->getPostStats1()->statRead(merged);
                     readPassed++;
                     mergedCount++;
                 }
                 delete merged;
-            } else if(!mOptions->merge.discardUnmerged){
+                mergeProcessed = true;
+            } else if(mOptions->merge.includeUnmerged){
                 int result1 = mFilter->passFilter(r1);
                 config->addFilterResult(result1, 1);
                 if(result1 == PASS_FILTER) {
-                    singleOutput += r1->toString();
+                    mergedOutput += r1->toString();
                     config->getPostStats1()->statRead(r1);
                 }
 
                 int result2 = mFilter->passFilter(r2);
                 config->addFilterResult(result2, 1);
                 if(result2 == PASS_FILTER) {
-                    singleOutput += r2->toString();
+                    mergedOutput += r2->toString();
                     config->getPostStats1()->statRead(r2);
                 }
                 if(result1 == PASS_FILTER && result2 == PASS_FILTER )
                     readPassed++;
+                mergeProcessed = true;
             }
-        } else {
+        }
+
+        if(!mergeProcessed) {
 
             int result1 = mFilter->passFilter(r1);
             int result2 = mFilter->passFilter(r2);
@@ -364,7 +444,7 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
 
             if( r1 != NULL &&  result1 == PASS_FILTER && r2 != NULL && result2 == PASS_FILTER ) {
                 
-                if(mOptions->outputToSTDOUT) {
+                if(mOptions->outputToSTDOUT && !mOptions->merge.enabled) {
                     singleOutput += r1->toString() + r2->toString();
                 } else {
                     outstr1 += r1->toString();
@@ -372,10 +452,34 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
                 }
 
                 // stats the read after filtering
-                config->getPostStats1()->statRead(r1);
-                config->getPostStats2()->statRead(r2);
+                if(!mOptions->merge.enabled) {
+                    config->getPostStats1()->statRead(r1);
+                    config->getPostStats2()->statRead(r2);
+                }
 
                 readPassed++;
+            } else if( r1 != NULL &&  result1 == PASS_FILTER) {
+                if(mUnpairedLeftWriter) {
+                    unpairedOut1 += r1->toString();
+                    if(mFailedWriter)
+                        failedOut += or2->toStringWithTag(FAILED_TYPES[result2]);
+                } else {
+                    if(mFailedWriter) {
+                        failedOut += or1->toStringWithTag("paired_read_is_failing");
+                        failedOut += or2->toStringWithTag(FAILED_TYPES[result2]);
+                    }
+                }
+            } else if( r2 != NULL && result2 == PASS_FILTER) {
+                if(mUnpairedLeftWriter || mUnpairedRightWriter) {
+                    unpairedOut2 += r2->toString();
+                    if(mFailedWriter)
+                        failedOut += or1->toStringWithTag(FAILED_TYPES[result1]);
+                } else {
+                    if(mFailedWriter) {
+                        failedOut += or1->toStringWithTag(FAILED_TYPES[result1]);
+                        failedOut += or2->toStringWithTag("paired_read_is_failing");
+                    }
+                }
             }
         }
 
@@ -388,35 +492,73 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
             delete r2;
     }
     // if splitting output, then no lock is need since different threads write different files
-    if(!mOptions->split.enabled)
+    if(!mOptions->split.enabled) 
         mOutputMtx.lock();
     if(mOptions->outputToSTDOUT) {
         // STDOUT output
-        fwrite(singleOutput.c_str(), 1, singleOutput.length(), stdout);
+        // if it's merging mode, write the merged reads to STDOUT
+        // otherwise write interleaved single output
+        if(mOptions->merge.enabled)
+            fwrite(mergedOutput.c_str(), 1, mergedOutput.length(), stdout);
+        else
+            fwrite(singleOutput.c_str(), 1, singleOutput.length(), stdout);
     } else if(mOptions->split.enabled) {
         // split output by each worker thread
         if(!mOptions->out1.empty())
             config->getWriter1()->writeString(outstr1);
         if(!mOptions->out2.empty())
             config->getWriter2()->writeString(outstr2);
-    } else {
-        // normal output by left/right writer thread
-        if(mRightWriter && mLeftWriter) {
-            // write PE
-            char* ldata = new char[outstr1.size()];
-            memcpy(ldata, outstr1.c_str(), outstr1.size());
-            mLeftWriter->input(ldata, outstr1.size());
+    } 
 
-            char* rdata = new char[outstr2.size()];
-            memcpy(rdata, outstr2.c_str(), outstr2.size());
-            mRightWriter->input(rdata, outstr2.size());
-        } else if(mLeftWriter) {
-            // write singleOutput
-            char* ldata = new char[singleOutput.size()];
-            memcpy(ldata, singleOutput.c_str(), singleOutput.size());
-            mLeftWriter->input(ldata, singleOutput.size());
+    if(mMergedWriter && !mergedOutput.empty()) {
+        // write merged data
+        char* mdata = new char[mergedOutput.size()];
+        memcpy(mdata, mergedOutput.c_str(), mergedOutput.size());
+        mMergedWriter->input(mdata, mergedOutput.size());
+    }
+
+    if(mFailedWriter && !failedOut.empty()) {
+        // write failed data
+        char* fdata = new char[failedOut.size()];
+        memcpy(fdata, failedOut.c_str(), failedOut.size());
+        mFailedWriter->input(fdata, failedOut.size());
+    }
+
+    // normal output by left/right writer thread
+    if(mRightWriter && mLeftWriter && (!outstr1.empty() || !outstr2.empty())) {
+        // write PE
+        char* ldata = new char[outstr1.size()];
+        memcpy(ldata, outstr1.c_str(), outstr1.size());
+        mLeftWriter->input(ldata, outstr1.size());
+
+        char* rdata = new char[outstr2.size()];
+        memcpy(rdata, outstr2.c_str(), outstr2.size());
+        mRightWriter->input(rdata, outstr2.size());
+    } else if(mLeftWriter && !singleOutput.empty()) {
+        // write singleOutput
+        char* ldata = new char[singleOutput.size()];
+        memcpy(ldata, singleOutput.c_str(), singleOutput.size());
+        mLeftWriter->input(ldata, singleOutput.size());
+    }
+    // output unpaired reads
+    if (!unpairedOut1.empty() || !unpairedOut2.empty()) {
+        if(mUnpairedLeftWriter && mUnpairedRightWriter) {
+            // write PE
+            char* unpairedData1 = new char[unpairedOut1.size()];
+            memcpy(unpairedData1, unpairedOut1.c_str(), unpairedOut1.size());
+            mUnpairedLeftWriter->input(unpairedData1, unpairedOut1.size());
+
+            char* unpairedData2 = new char[unpairedOut2.size()];
+            memcpy(unpairedData2, unpairedOut2.c_str(), unpairedOut2.size());
+            mUnpairedRightWriter->input(unpairedData2, unpairedOut2.size());
+        } else if(mUnpairedLeftWriter) {
+            char* unpairedData = new char[unpairedOut1.size() + unpairedOut2.size() ];
+            memcpy(unpairedData, unpairedOut1.c_str(), unpairedOut1.size());
+            memcpy(unpairedData + unpairedOut1.size(), unpairedOut2.c_str(), unpairedOut2.size());
+            mUnpairedLeftWriter->input(unpairedData, unpairedOut1.size() + unpairedOut2.size());
         }
     }
+
     if(!mOptions->split.enabled)
         mOutputMtx.unlock();
 
@@ -653,6 +795,14 @@ void PairEndProcessor::consumerTask(ThreadConfig* config)
             mLeftWriter->setInputCompleted();
         if(mRightWriter)
             mRightWriter->setInputCompleted();
+        if(mUnpairedLeftWriter)
+            mUnpairedLeftWriter->setInputCompleted();
+        if(mUnpairedRightWriter)
+            mUnpairedRightWriter->setInputCompleted();
+        if(mMergedWriter)
+            mMergedWriter->setInputCompleted();
+        if(mFailedWriter)
+            mFailedWriter->setInputCompleted();
     }
     
     if(mOptions->verbose) {
